/*
 * Parser definitions.
 *
 * Copyright (C) 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include "types.h"

/* Value copying, avoiding problems with retaining values in structures. */

void *copy(void *value);

/* Specialised copying. */

#define copy_attribute(in) (struct attribute *) copy(&in);
#define copy_identifier(in) (struct identifier *) copy(&in);
#define copy_include(in) (struct include *) copy(&in);
#define copy_interface(in) (struct interface *) copy(&in);
#define copy_parameter(in) (struct parameter *) copy(&in);
#define copy_signature(in) (struct signature *) copy(&in);

/* Generalised copying to structures with tail members. */

#define copy_tail(type, out, in) out.tail = (struct type *) copy(&in)
