/*
 * Provide a calculation object for local use.
 *
 * Copyright (C) 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/types.h>

#include <math.h>

#include "calc_interface.h"



/* Component methods. */

static long calc_add(ref_Calc _self, int left, int right, int *result)
{
  (void) _self;
  *result = left + right;
  return L4_EOK;
}

static long calc_subtract(ref_Calc _self, int left, int right, int *result)
{
  (void) _self;
  *result = left - right;
  return L4_EOK;
}

static long calc_multiply(ref_Calc _self, int left, int right, int *result)
{
  (void) _self;
  *result = left * right;
  return L4_EOK;
}

static long calc_divide(ref_Calc _self, int numerator, int denominator, int *result)
{
  (void) _self;
  *result = numerator / denominator;
  return L4_EOK;
}

static long calc_pow(ref_Calc _self, double base, double exponent, double *result)
{
  (void) _self;
  *result = powf(base, exponent);
  return L4_EOK;
}



/* Interface for this component. */

iface_Calc local_iface_Calc = {
  .add=calc_add,
  .subtract=calc_subtract,
  .multiply=calc_multiply,
  .divide=calc_divide,
  .pow=calc_pow
  };
